import java.io.File;
import java.io.IOException;

import com.objectxp.msg.GsmSmsService;
import com.objectxp.msg.MessageEvent;
import com.objectxp.msg.MessageEventListener;
import com.objectxp.msg.MessageException;
import com.objectxp.msg.SmsMessage;
import com.objectxp.msg.SmsService;

/**
 * This examples demonstrates how to send and receive SMS using jSMS
 */
public class SendAndReceive implements MessageEventListener
{
  public void handleMessageEvent(MessageEvent event) 
  {
    if (event.getType() == MessageEvent.MESSAGE_RECEIVED) {
      System.out.println("Received message is: " + event.getMessage());
    } else {
      System.out.println("Event received: "+event);
    }
  }
  
  public static void main(String args[]) throws IOException, MessageException 
  {
    if(args.length != 2 ) {
      System.err.println("Usage: ReceiveExample <config-file> <recipient>");
      System.exit(1);
    }
    
    File config = new File(args[0]);
    SmsService service = new GsmSmsService();
    
    try {
      service.init(config);
      service.addMessageEventListener(new ReceiveExample());
      // receiving messages...
      service.connect();
      service.startReceiving();
      
      // send message
      SmsMessage msg = new SmsMessage();
      msg.setRecipient(args[1]);
      msg.setMessage("Hello World");
      service.sendMessage(msg);
       
      // keep receiving messages...      
      System.out.print("Press any key to stop receiving messages");
      System.in.read();
      
      service.stopReceiving();
      service.disconnect();
    } finally {
      service.destroy();
    }
  }
}
