import java.io.File;
import java.io.IOException;

import com.objectxp.msg.MessageEvent;
import com.objectxp.msg.MessageEventListener;
import com.objectxp.msg.MessageException;
import com.objectxp.msg.SmppSmsService;
import com.objectxp.msg.SmsMessage;
import com.objectxp.msg.WindowingService;

/**
 * This example demonstrates how to send a Text SMS using jSMS with Windowing 
 * enabled. 
 */
public class Windowing implements MessageEventListener
{
  private int mSent=0;	

  public WindowingService mService;
  
  public void handleMessageEvent(MessageEvent event) {
	  
	  if(event.getType() == MessageEvent.MESSAGE_SENT) {
		  mSent++;
		  System.out.println("-----------> message sent");
		  System.out.println("Pending operations: "+mService.getNumberOfPendingMessages());
		  if(mSent == 5) {
			  synchronized(this) {
				  notify();
			  }
			  System.out.println("All messages sent.");
		  }
	  }
		
	}

public static void main(String[] args) throws IOException, MessageException, IllegalAccessException 
  {
	
	if(args.length != 2 ) {
      System.err.println("Usage: Windowing <config-file> <recipient>");
      System.exit(1);
    }
    
    // create Windowing example
    Windowing winEx = new Windowing();
    
    // Create and initialize the SmsService 
    SmppSmsService service = new SmppSmsService();
    winEx.mService = service;
    service.init(new File(args[0]));
   
    
    
    // Set the window size for this service. 
    // To disable windowing, set the window size to 1. 
    // The window size can only be changed if the SmsService is not currently connected to the SMSC.
    service.setWindowSize(5);
    service.addMessageEventListener(winEx);
    // connect, send message and disconnect 
    try {
      service.connect();
       
      service.startReceiving();
      /*
       * This loop sends the same message 5 times to the same receiver. This 
       * demonstrates how to use the "getNumberOfPendingMessages" operation.
       */
      for (int i=1; i<=5; i++){
          // create message
          SmsMessage msg = new SmsMessage();
          msg.setRecipient(args[1]);
          msg.setMessage("Hello World #"+i);
          msg.setSender("0791234567");
          service.sendMessage(msg);
          // Returns the number of pending (unprocessed messages). 
          // You may use this method to check if all outgoing messages have been acknowledged 
          // by the SMSC prior to calling disconnect() on the SmsService.
          System.out.println("Pending operations: "+service.getNumberOfPendingMessages());
      }
      try {
    	  synchronized(winEx) {
              // wait until all messages have been sent
    		  winEx.wait(20000);
    	  }
      } catch (InterruptedException e) {
		e.printStackTrace();
      }
      service.stopReceiving();
      service.disconnect();
    } catch (Exception e) {
		e.printStackTrace();
      } 
    finally {
      service.destroy(); 
    }
  }
}
