package mms.mm7.webapp;

import java.io.FileInputStream;
import java.io.InputStream;
import java.util.Iterator;
import java.util.Vector;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import com.objectxp.mms.MMSDeliveryReport;
import com.objectxp.mms.MMSListener;
import com.objectxp.mms.MMSNotification;
import com.objectxp.mms.MMSReadReport;
import com.objectxp.mms.MMSStatus;
import com.objectxp.mms.MMSService;
import com.objectxp.mms.MMSServiceFactory;
import com.objectxp.mms.message.MMSMessage;
import com.objectxp.mms.protocol.MM7ReceiverServlet;

/**
 * Simple example that demonstrates how to receive multimedia-messages and 
 * reports using the MM7 MMS Service implementation in a J2EE Web application.
 * <p>
 * The servlet must be configured in the Web Application Descriptor (web.xml)
 * and the URL of the servlet must be announced to the MM7 MMSC operator.
 * </p>
 * The servlet will display a simple text form which can be used to send
 * a multimedia-message. Incoming and outgoing messages will be added to a List
 * and will also be displayed. 
 */
public class MyMM7Servlet extends MM7ReceiverServlet implements MMSListener
{
  private static final String DEFAULT_CONFIG = "/WEB-INF/mms.properties";
  private static final long serialVersionUID = 4561367956571536193L;
  private static final String MMS_CONFIG = "mms.config";
  
  private Vector incomingMessages = new Vector();
  private Vector sentMessages = new Vector();
  private MMSService mmsService;

  /**
   * Returns a MMSService implementation. If required, constructs the Service
   * using the MMS Service Factory.
   */
  public MMSService getService() {
    return mmsService;
  }

  /**
   * Get the default MMS service and connect to the MMSC upon initialization.
   */
  public void init(ServletConfig config) throws ServletException 
  {
    super.init(config);
    String configFile = config.getInitParameter(MMS_CONFIG);
    InputStream in = null;
    try {
      if(configFile!=null) {
        in=new FileInputStream(configFile);
      } else {
        in=config.getServletContext().getResourceAsStream(DEFAULT_CONFIG);
        if(in == null) {
          throw new ServletException("No configuration configured and unable " +
                "to load default configuration from "+DEFAULT_CONFIG);
          
        }
      }
      MMSServiceFactory factory = MMSServiceFactory.createFactory(in);
      in.close();
      mmsService = factory.getDefaultService();
    } catch (Exception e) {
      throw new ServletException("Unable to create MMS Service.", e);
    }
    try {
      mmsService.connect();
    } catch(Exception e) {
      throw new ServletException("Unable to connect to MMSC.", e);
    }
    // Install a listener for incoming messages and reports
    mmsService.setListener(this);
    config.getServletContext().setAttribute(this.getClass().getName(), this);
  }

  /**
   * Disconnect from the MMSC...
   */
  public void destroy() {
    if(mmsService != null){
      try {
        mmsService.disconnect();
      } catch(Exception e) {
        log("Warning: Unable to disconnect from MMSC.", e);
      }
      mmsService=null;
    }
  }

  public MMSStatus handleIncommingMessage(MMSService service, MMSMessage message) {
    log("Incoming message from "+message.getFrom());
    incomingMessages.add(message);
    return MMSStatus.SUCCESS;
    
  }

  public MMSStatus handleReadReport(MMSService service, MMSReadReport report) {
    log("Incoming read report: "+report);
    return MMSStatus.SUCCESS;
  }

  public MMSStatus handleDeliveryReport(MMSService service, MMSDeliveryReport report) {
    log("Incoming delivery report: "+report);
    return MMSStatus.SUCCESS;
  }

  public void handleNotification(MMSService service, MMSNotification notification) {}
  
  public Iterator getIncomingMessages() {
    return incomingMessages.iterator();
  }
  
  public Iterator getSentMessages() {
    return sentMessages.iterator();
  }
  
  public void addSentMessage(MMSMessage msg) {
    sentMessages.add(msg);
  }
}
