/**
 * JWAP - A Java Implementation of the WAP Protocols
 * Copyright (C) 2001-2004 Niko Bender
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package net.sourceforge.jwap.wtp.pdu;

import net.sourceforge.jwap.util.BitArrayOutputStream;
import net.sourceforge.jwap.util.Utils;


/**
 * This Class represents a Negative Acknowledgement PDU.
 */
public class CWTPNack extends CWTPPDU {
  
    private short[] missingPackets;

    /**
     * @param TID the Transaction ID according to the spec
     */
    public CWTPNack(int TID, short[] missingPackets) {
        super(TID, PDU_TYPE_NEG_ACK);
        this.missingPackets=missingPackets;
    }

    /**
     * encodes the PDU according to the WTP spec
     *
     * @return encoded bytes
     */
    public byte[] toByteArray() {
        BitArrayOutputStream result = new BitArrayOutputStream();
        result.write(CON);
        result.write(pduType, 4);
        result.write(0,2); // Two reserved pids
        result.write(RID);
        result.write(TID, 16);
        
        result.write(missingPackets==null?0:missingPackets.length,8);
        if(missingPackets != null) {
          for( int i=0; i<missingPackets.length; i++) {
            result.write(missingPackets[i],8);
          }
        }
        if (payload != null) {
          result.write(payload);
      }
        return result.toByteArray();
    }

    
    /**
     * constructs a string representation of the object
     * invluding all fields.
     *
     * @return The constructed String with debug information
     */
    public String toString() {
      StringBuffer sb = new StringBuffer();
      sb.append("[WTP Nack PDU\n");
      sb.append(" CON: ").append(CON?1:0).append("\n");
      sb.append(" RID: ").append(RID?1:0).append("\n");
      sb.append(" TID: ").append(Utils.hexString(TID)).append("\n");
      sb.append(" Number of missing Packets=").append(missingPackets==null?0:missingPackets.length);
      if(missingPackets != null) {
        sb.append(":");
        for( int i=0; i<missingPackets.length; i++) {
          sb.append(" ").append(missingPackets[i]);
        }
      }
      sb.append("\n");
      if(payload != null) {
        sb.append(Utils.hexDump("  PAYLOAD: ",payload));
      }
      sb.append("]");
      return sb.toString();
    }
}
